@@ -4,17 +4,18 @@
 #'
 #'
 #'
-#'
-#' @param level Character. The level of aggregation of Invalsi census data. Either \code{"NUTS-3"}, \code{"Province"}, \code{"LAU"}, \code{"Municipality"}. \code{"LAU"} by default.
+#' @param multiple_out Logical. Wheter keeping
+#' multiple dataframes as outputs (thus overriding the \code{level} argument) or not.
+#' We advise to keep this option active in order not to lose information. \code{TRUE} by default.
+#' @param level Character. The level of aggregation of Invalsi census data, such as \code{"NUTS-3"}, \code{"Province"} for provinces; or \code{"LAU"}, \code{"Municipality"} for muncipalities.
+#' Only useful when \code{multiple_out == FALSE}. \code{"LAU"} by default.
 #' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
 #' @param show_col_types Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.
 #' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
-#' @param multiple_out Logical. Wheter keeping
-#' multiple dataframes as outputs (thus overriding the \code{level} argument) or not.
 #' \code{TRUE} by default.
 #' @param category Logical. Whether to focus on a specific category of students participating to the census survey. Warning: experimental. \code{FALSE} by default.
 #'
-#' @return Unless \code{multiple_out == TRUE}, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
+#' @return If \code{multiple_out == FALSE}, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
 #' Otherwise, a list including objects of the aforementioned classes
 #'
 #' @source  <https://serviziostatistico.invalsi.it/en/archivio-dati/?_sft_invalsi_ss_data_collective=open-data>
@@ -50,7 +51,8 @@
 #'
 #' @examples
 #' \donttest{
-#' Get_Invalsi_IS(level = "NUTS-3", autoAbort = TRUE, verbose = FALSE)
+#' Data_Invalsi <- Get_Invalsi_IS(level = "NUTS-3", autoAbort = TRUE, verbose = FALSE)
+#' Data_Invalsi$Province_data
 #' }
 #'
 #'
@@ -63,111 +65,99 @@ Get_Invalsi_IS <- function(level = "LAU", verbose = TRUE, show_col_types = FALSE
   if(!Check_connection(autoAbort)) return(NULL)
 
   starttime <- Sys.time()
-  if (verbose)
-    cat("Retrieving Invalsi census data")
-  if (toupper(level) %in% c("MUNICIPALITY", "MUN", "LAU", "NUTS-4")){
-    nCol <- 10
-    if(!multiple_out) cat (" for municipalities")
-    url.invalsi <- "https://serviziostatistico.invalsi.it/invalsi_ss_data/dati-comunali-di-popolazione-comune-del-plesso/"
-    name_pattern <- "report_comuni_plessi"
-  } else if (toupper(level) %in%c("PROVINCE", "PROV", "NUTS-3") ){
-    nCol <- 11
-    if (!multiple_out) cat("for provinces")
-    url.invalsi <- "https://serviziostatistico.invalsi.it/invalsi_ss_data/dati-provinciali-di-popolazione/"
-    name_pattern <- "matrice_medie_provinciali"
-  }
-  if (verbose) cat(" \n")
+  if (verbose) cat("Retrieving Invalsi census data ... \n")
+  url.invalsi <- "https://serviziostatistico.invalsi.it/en/invalsi_ss_data/punteggi-e-percentuale-di-studenti-nei-livelli-di-competenza-per-ripartizioni-territoriali-e-caratteristiche-di-contesto/"
 
-  homepage <- NULL
-  massive_DB <- FALSE
   homepage <- tryCatch({
     xml2::read_html(url.invalsi)
   }, error = function(e){
     NULL
   })
-  if(is.null(homepage)){
-    url.invalsi <- "https://serviziostatistico.invalsi.it/en/invalsi_ss_data/punteggi-e-percentuale-di-studenti-nei-livelli-di-competenza-per-ripartizioni-territoriali-e-caratteristiche-di-contesto/"
-    homepage <- tryCatch({
-      xml2::read_html(url.invalsi)
-    }, error = function(e){
-      NULL
-    })
-    if(!is.null(homepage)){
-      massive_DB <- TRUE
-      name_pattern <- "report_generale"
-    } else {
-      message("Invalsi webpage not found. We apologise for the inconvenience. If the problem persists, please contact the maintainer.")
+  if(!is.null(homepage)){
+    name_pattern <- "report_generale"
+  } else {
+    message("It seems INVALSI changed address. \n Please contact the maintainer and ask him to fix.")
+    return(NULL)
+  }
+
+  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href")
+  # Warning: this works for the current website architecture - may
+  # change in the future
+  links <- links[grepl(paste0(name_pattern), links)]
+  link <- unique(links[grepl("zip", links)])
+  if(length(link)==0L){
+    message("It seems INVALSI changed address. \n Please contact the maintainer and ask him to fix.")
+    return(NULL)
+  }
+  base.url <- dirname(url.invalsi)
+  file.url <- xml2::url_absolute(link, base.url)
+  temp1 <- tempfile(fileext=".zip")
+  timeout.default <- options("timeout")$timeout
+  on.exit(options(timeout = timeout.default))
+  options(timeout = max(timeout.default, 180))
+
+  attempt <- 0
+  x <- NULL
+  while(is.null(x)){
+    x <- tryCatch({
+      utils::download.file(url = file.url, destfile = temp1, mode = "wb", quiet = TRUE)},
+      error = function(e) {
+        NULL
+      })
+    if(is.null(x)){
+      attempt <- attempt + 1
+      message("Error in file downloading, ", 10-attempt, " attempts left.\n")
+    }
+    if(attempt >= 10){
+      unlink(temp1, recursive = TRUE, force = TRUE)
+      message("Unable to download data; max attempts reached.")
+      message("Abort. Please contact the maintiner and ask him to fix.\n")
       return(NULL)
     }
   }
 
-  Invalsi_IS <- NULL
-  status <- 0
-  attempt <- 0
-  while(status != 200 && attempt <= 10){
-    links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href")
-    if(!massive_DB){
-      link <- unique(links[grepl(paste0(
-        "(?=.*", name_pattern, ")(?=.*\\.csv$)(?!.*", "livelli", ")"), links, perl = TRUE)])
-    } else {
-      link <- unique(links[grepl(paste0(
-        "^(?!.*tracciato)(?=.*", name_pattern, ")(?=.*\\.csv)"), links, perl = TRUE)])
-    }
-    if(verbose){
-      response <- tryCatch({
-        httr::GET(link, httr::progress())
-      }, error = function(e) {
-        message("Error occurred during scraping, attempt repeated ... \n")
-        NULL
-      })
-    } else {
-      response <- tryCatch({
-        httr::GET(link)
-      }, error = function(e) {
-        message("Error occurred during scraping, attempt repeated ... \n")
-        NULL
-      })
-    }
+  options(timeout = timeout.default)
 
-    status <- response$status_code
-    if(is.null(response)){
-      status <- 0
-    }
-    if(status != 200) {
-      attempt <- attempt + 1
-      if(!autoAbort){
-        message("Error occurred; connection exited with status ", status, " ; ", 10 - attempt, " attempts left \n",
-                "To abort the operation, press `A`; to hold on press any key \n")
-        holdOn <- readline(prompt = "    ")
-        if(holdOn == "A") {
-          message("You chose to abort the operation. We apologise for the inconvenience.")
+  attempt <- 0
+  Invalsi_IS <- NULL
+  while(is.null(Invalsi_IS)){
+    zip.content <- utils::unzip(temp1, list = TRUE)
+    content.csv <- zip.content[which(grepl("\\.csv$", zip.content$Name, ignore.case = TRUE)),]
+    if(nrow(content.csv) > 0L){
+      csv.max <- which.max(zip.content$Length)
+      filename <- zip.content$Name[csv.max]
+      con <- unz(temp1, filename)
+      Invalsi_IS <- tryCatch({
+        readr::read_delim( con, delim = ";", na = c("", " ", "N.A."),
+                           locale = readr::locale(decimal_mark = ",",
+                                                  encoding = "ISO-8859-1"),
+                           show_col_types = show_col_types )},
+        error = function(e){
+          message("For some reason, R cannot open the .csv file... \n")
           return(NULL)
-        }
-      } else return(NULL)
+        })
+    }
+    if(is.null(Invalsi_IS)) {
+      message("Retry operation. ", 10-attempt, " attempts left... \n")
+      attempt <- attempt + 1
+    }
+    if(attempt >= 10) {
+      message("Impossible to read Invalsi file; max attempts reached. Abort. \n",
+      "Please contact the maintainer and ask him to fix. \n")
+      unlink(temp1, recursive = TRUE, force = TRUE)
+
+      return(NULL)
     }
-  }
-  if(status == 200){
-    if(verbose) cat("Encoding raw content in UTF-8 \n")
-    content.UTF8 <- iconv(rawToChar(response$content), from = "ISO-8859-1", to = "UTF-8")
-    Invalsi_IS <- readr::read_delim(content.UTF8, delim = ";",
-                                    show_col_types = show_col_types, locale = readr::locale(decimal_mark = ","))
-  } else {
-    message("Maximum (10) attempts exceeded. We apologise for the inconvenience.")
-    return(NULL)
-  }
-  if(!massive_DB){
-    Invalsi_IS[,c((nCol-4):nCol)] <- Invalsi_IS[,c((nCol-4):nCol)] %>%
-      apply(MARGIN = 2, FUN = function(x){as.numeric(gsub(",", ".", x))} )
   }
 
   names(Invalsi_IS) <- names(Invalsi_IS) %>%
     stringr::str_replace_all("Aggregato_territoriale","Territorial_aggregate") %>%
-    stringr::str_replace_all("aggregazione","Misc_level") %>%
+    stringr::str_replace_all("aggregazione","Level") %>%
     stringr::str_replace_all("Denominazione", "Description") %>%
     stringr::str_replace_all("Codice_provincia", "Province_code") %>%
     stringr::str_replace_all("Nome_provincia", "Province_description") %>%
     stringr::str_replace_all("Sigla_provincia", "Province_initials") %>%
-    stringr::str_replace_all("id_comune", "Municipality_code") %>%
+    stringr::str_replace_all("id", "ID") %>%
     stringr::str_replace_all("Comune", "Municipality_description") %>%
     stringr::str_replace_all("Grado", "Grade") %>%
     stringr::str_replace_all("Materia", "Subject") %>%
@@ -191,89 +181,83 @@ Get_Invalsi_IS <- function(level = "LAU", verbose = TRUE, show_col_types = FALSE
     stringr::str_replace_all("Quarto_Livello", "Fourth_level") %>%
     stringr::str_replace_all("Quinto_Livello", "Fifth_level") %>%
     stringr::str_replace_all("Perc_traguardi", "Targets_percentage") %>%
-    stringr::str_replace_all("Percentuale_partecipazione", "Students_coverage") %>%
-    stringr::str_replace_all("^id$", "ID")
+    stringr::str_replace_all("Percentuale_partecipazione", "Students_coverage")
 
   Invalsi_IS$Subject <- Invalsi_IS$Subject %>%
     stringr::str_replace_all("Matematica", "Mathematics") %>%
     stringr::str_replace_all("Italiano", "Italian") %>%
     stringr::str_replace_all("Inglese", "English")
 
-  if(!massive_DB){
-    if (toupper(level) %in% c("MUNICIPALITY", "LAU", "NUTS-4")){
-      Invalsi_IS <- Invalsi_IS %>%
-        dplyr::mutate(Municipality_code = sprintf("%06d", .data$Municipality_code)) %>%
-        dplyr::mutate(Municipality_description = stringr::str_to_title(.data$Municipality_description))
-    } else {
-      Invalsi_IS <- Invalsi_IS %>%
-        dplyr::mutate(Province_description = stringr::str_to_title(.data$Province_description))
-    }
-  } else{
+
+
+
+  Invalsi_IS <- Invalsi_IS %>%
+    dplyr::mutate(Description = stringr::str_to_title(.data$Description))
+
+  # This is a loss in information next package versions shall overcome:
+  if(!category){
     Invalsi_IS <- Invalsi_IS %>%
-      dplyr::mutate(Description = stringr::str_to_title(.data$Description))
-
-    # This is a loss in information next package versions shall overcome:
-    if(!category){
-      Invalsi_IS <- Invalsi_IS %>%
-        dplyr::filter(.data$Misc_level == "Totale") %>%
-        dplyr::select(-.data$Misc_level)
-    } else{
-      categories <- unique(Invalsi_IS$Misc_level)
-      categories_mat <- apply(cbind(c(1:length(categories)), categories), 1,
-                              function(X) paste0(X, collapse = ": "))
-      cat("Please, choose one or more numbers \n corresponding to the following categories;
-          \n use the semicolon (;) as separator \n",
-          paste(categories_mat, collapse = "\n"))
-      nn_categories <- as.numeric(unlist(strsplit(readline(prompt = "  > "), split = ";")))
-      categories_in <- categories[nn_categories]
-      Invalsi_IS <- Invalsi_IS %>%
-        dplyr::filter(.data$Misc_level %in% categories_in) %>%
-        dplyr::rename( Category = .data$Misc_level)
+      dplyr::filter(.data$Level == "Totale") %>%
+      dplyr::select(-.data$Level)
+  } else{
+    categories <- unique(Invalsi_IS$Level)
+    categories_mat <- apply(cbind(c(1:length(categories)), categories), 1,
+                            function(X) paste0(X, collapse = ": "))
+    cat("Please, choose one or more numbers \n corresponding to the following categories;
+         \n use the semicolon (;) as separator \n",
+         paste(categories_mat, collapse = "\n"))
+    nn_categories <- as.numeric(unlist(strsplit(readline(prompt = "  > "), split = ";")))
+    categories_in <- categories[nn_categories]
+    Invalsi_IS <- Invalsi_IS %>%
+      dplyr::filter(.data$Level %in% categories_in) %>%
+      dplyr::rename( Category = .data$Level)
     }
 
 
-    dd_mun <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Comune plesso") %>%
-      dplyr::rename(Municipality_code = .data$ID) %>%
-      dplyr::rename(Municipality_description = .data$Description) %>%
-      dplyr::select(-.data$Territorial_aggregate)%>%
-      dplyr::mutate(Municipality_code = sprintf("%06d", .data$Municipality_code))
-    dd_prov <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Provincia") %>%
-      dplyr::rename(Province_code = .data$ID) %>%
-      dplyr::rename(Province_description = .data$Description) %>%
-      dplyr::select(-.data$Territorial_aggregate)
-    dd_reg <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Regione") %>%
-      dplyr::rename(Region_code = .data$ID) %>%
-      dplyr::rename(Region_description = .data$Description) %>%
-      dplyr::select(-.data$Territorial_aggregate)
-    dd_LLS <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Sistemi locali del lavoro (SLL)") %>%
-      dplyr::rename(LLS_code = .data$ID) %>%
-      dplyr::rename(LLS_description = .data$Description) %>%
-      dplyr::select(-.data$Territorial_aggregate)
-    dd_InnerAreas2014 <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Aree interne SNAI 2014") %>%
-      dplyr::rename(Inner_area_code = .data$ID) %>%
-      dplyr::rename(Inner_area_description = .data$Description) %>%
-      dplyr::select(-.data$Territorial_aggregate) # Not sure this is relevant
-    dd_InnerAreas2021 <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Aree interne SNAI 2021") %>%
-      dplyr::rename(Inner_area_code = .data$ID) %>%
-      dplyr::rename(Inner_area_description = .data$Description) %>%
-      dplyr::select(-.data$Territorial_aggregate)
-    dd_macroarea <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Area geografica") %>%
-      dplyr::rename(Macroarea_code = .data$ID) %>%
-      dplyr::rename(Macroarea_description = .data$Description) %>%
-      dplyr::select(-.data$Territorial_aggregate) %>%
-      dplyr::mutate(dplyr::across(
-        .data$Macroarea_description, ~ stringr::str_replace_all(.x, c(
-          "Nord Ovest" = "North-West",
-          "Nord Est" = "North-East",
-          "Centro" = "Center",
-          "Sud E Isole" = "South and Islands",
-          "Sud" = "South"))))
+  dd_mun <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Comune plesso") %>%
+    dplyr::rename(Municipality_code = .data$ID) %>%
+    dplyr::rename(Municipality_description = .data$Description) %>%
+    dplyr::select(-.data$Territorial_aggregate)%>%
+    dplyr::mutate(Municipality_code = sprintf("%06d", .data$Municipality_code))
+  dd_prov <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Provincia") %>%
+    dplyr::rename(Province_code = .data$ID) %>%
+    dplyr::rename(Province_description = .data$Description) %>%
+    dplyr::select(-.data$Territorial_aggregate)
+  dd_reg <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Regione") %>%
+    dplyr::rename(Region_code = .data$ID) %>%
+    dplyr::rename(Region_description = .data$Description) %>%
+    dplyr::select(-.data$Territorial_aggregate)
+  dd_LLS <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Sistemi locali del lavoro (SLL)") %>%
+    dplyr::rename(LLS_code = .data$ID) %>%
+    dplyr::rename(LLS_description = .data$Description) %>%
+    dplyr::select(-.data$Territorial_aggregate)
+  dd_InnerAreas2014 <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Aree interne SNAI 2014") %>%
+    dplyr::rename(Inner_area_code = .data$ID) %>%
+    dplyr::rename(Inner_area_description = .data$Description) %>%
+    dplyr::select(-.data$Territorial_aggregate) # Not sure this is relevant
+  dd_InnerAreas2021 <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Aree interne SNAI 2021") %>%
+    dplyr::rename(Inner_area_code = .data$ID) %>%
+    dplyr::rename(Inner_area_description = .data$Description) %>%
+    dplyr::select(-.data$Territorial_aggregate)
+  dd_macroarea <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Area geografica") %>%
+    dplyr::rename(Macroarea_code = .data$ID) %>%
+    dplyr::rename(Macroarea_description = .data$Description) %>%
+    dplyr::select(-.data$Territorial_aggregate) %>%
+    dplyr::mutate(dplyr::across(
+      .data$Macroarea_description, ~ stringr::str_replace_all(.x, c(
+         "Nord Ovest" = "North-West",
+        "Nord Est" = "North-East",
+        "Centro" = "Center",
+        "Sud E Isole" = "South and Islands",
+        "Sud" = "South"))))
 
-    if(multiple_out){
-      Invalsi_IS <- list(Municipality_data = dd_mun, Province_data = dd_prov,
-                         Region_data = dd_reg, LLS_data = dd_LLS, Inner_Areas_2014_data = dd_InnerAreas2014,
-                         Inner_Areas_2021_data = dd_InnerAreas2021, Macroarea_data = dd_macroarea)
-    } else{
+  if(multiple_out){
+    Invalsi_IS <- list(Municipality_data = dd_mun, Province_data = dd_prov,
+                       Region_data = dd_reg, LLS_data = dd_LLS,
+                       Inner_Areas_2014_data = dd_InnerAreas2014,
+                       Inner_Areas_2021_data = dd_InnerAreas2021,
+                       Macroarea_data = dd_macroarea)
+  } else{
       if (toupper(level) %in% c("MUNICIPALITY", "MUN", "LAU", "NUTS-4")){
         Invalsi_IS <- dd_mun
       } else if(toupper(level) %in% c("PROV", "PROVINCE", "NUTS-3")){
@@ -297,12 +281,15 @@ Get_Invalsi_IS <- function(level = "LAU", verbose = TRUE, show_col_types = FALSE
       } else if(toupper(level) %in% c("MACROAREA", "NUTS-1", "NCS", "NORTHCENTERSOUTH", "NORTHCENTRESOUTH")){
         Invalsi_IS <- dd_macroarea
       } else{
-        message("Level not recognised: `", level, "`; municipality data returned by default")
+        message("Level not recognised: `", level, "`; municipality data returned by default. Sorry. \n")
         Invalsi_IS <- dd_mun
       }
-    }
   }
 
+
+  unlink(temp1, recursive = TRUE, force = TRUE)
+
+
   endtime <- Sys.time()
   if(verbose) {
     cat(paste("Total running time to retrieve Invalsi", level, "data:",
