@@ -80,7 +80,7 @@ Get_School2mun <- function(Year = 2023, show_col_types = FALSE, verbose = TRUE,
   attempt <- 0
   while(is.null(homepage)){
     homepage <- tryCatch({
-      xml2::read_html(home.url)
+      httr::content(httr::GET(home.url))
     }, error = function(e){
       message("Cannot read the html. If the problem persists, please contact the maintainer.\n")
       return(NULL)
@@ -157,7 +157,7 @@ Get_School2mun <- function(Year = 2023, show_col_types = FALSE, verbose = TRUE,
         Please contact the maintainer, maybe it could help. \n")
         return(NULL)
       } else {
-        input_Registry1 <- readr::read_csv(rawToChar(response$content), show_col_types = FALSE)
+        input_Registry1 <- readr::read_csv(content, show_col_types = FALSE)
       }
     } else {
       message(paste("Wrong file type:", httr::http_type(response)) )
